/**
 * vim: set ts=4 :
 * =============================================================================
 * SourceMod Sample Extension
 * Copyright (C) 2004-2008 AlliedModders LLC.  All rights reserved.
 * =============================================================================
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License, version 3.0, as published by the
 * Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * As a special exception, AlliedModders LLC gives you permission to link the
 * code of this program (as well as its derivative works) to "Half-Life 2," the
 * "Source Engine," the "SourcePawn JIT," and any Game MODs that run on software
 * by the Valve Corporation.  You must obey the GNU General Public License in
 * all respects for all other code used.  Additionally, AlliedModders LLC grants
 * this exception to all derivative works.  AlliedModders LLC defines further
 * exceptions, found in LICENSE.txt (as of this writing, version JULY-31-2007),
 * or <http://www.sourcemod.net/license.php>.
 *
 * Version: $Id$
 */

#include "extension.h"
#include "iserver.h"
#include "CDetour/detours.h"
#include <inetchannel.h>
#include <iclient.h>

/**
 * @file extension.cpp
 * @brief Implement extension code here.
 */

CArena_RunTeamLogic g_ServerTools;		/**< Global singleton for extension's main interface */

SMEXT_LINK(&g_ServerTools);

CDetour *runTeamLogicDetour = NULL;
CDetour *PlayerByIndex = NULL;

IGameConfig *g_pGameConf = NULL;

CGlobalVars *globals = NULL;

int g_oldvalue = 0;

CDetourManager::DetourReturn RunTeamLogicDetour(void *thisptr)
{
	printf("arena detour triggered\n");
	g_oldvalue = globals->maxClients;
	globals->maxClients = 48;
	
	RETURN_DETOUR(CDetourManager::DetourReturn_Ignored);
}

void postArenaDetour()
{
	printf("arena hook\n");
	globals->maxClients = g_oldvalue;
	g_oldvalue = 0;
}

CDetourManager::DetourReturn UTIL_PlayerByIndex(int playerIndex)
{
	if (g_oldvalue && playerIndex > g_oldvalue)
	{
		RETURN_DETOUR_VALUE(CDetourManager::DetourReturn_Override, 0);
	}

	if (g_oldvalue)
	{
		globals->maxClients = g_oldvalue;
	}

	RETURN_DETOUR(CDetourManager::DetourReturn_Ignored);
}

void postUtilDetour()
{
	if (g_oldvalue)
	{
		globals->maxClients = 48;
	}
}

bool CArena_RunTeamLogic::SDK_OnMetamodLoad(ISmmAPI *ismm, char *error, size_t maxlen, bool late)
{
	globals = ismm->GetCGlobals();
	return true;
}

bool CArena_RunTeamLogic::SDK_OnLoad(char *error, size_t maxlength, bool late)
{
	char conf_error[255];
	if (!gameconfs->LoadGameConfigFile("runteamlogic", &g_pGameConf, conf_error, sizeof(conf_error)))
	{
		if (conf_error)
		{
			snprintf(error, maxlength, "Could not read runteamlogic.txt: %s", conf_error);
		}
		return false;
	}

	CDetourManager::Init(g_pSM->GetScriptingEngine(), g_pGameConf);

	g_postDetour = (void *)&postUtilDetour;
	g_nonMember = true;	
	PlayerByIndex = CDetourManager::CreateDetour((void *)&UTIL_PlayerByIndex, 1, "PlayerByIndex");
	if (PlayerByIndex == NULL)
	{
		return false;
	}
	
	PlayerByIndex->EnableDetour();

	g_nonMember = false;

	g_postDetour = (void *)&postArenaDetour;
	runTeamLogicDetour = CDetourManager::CreateDetour((void *)&RunTeamLogicDetour, 0, "RunTeamLogic");
	if (runTeamLogicDetour == NULL)
	{
		return false;
	}

	runTeamLogicDetour->EnableDetour();

	return true;
}

void CArena_RunTeamLogic::SDK_OnAllLoaded()
{
}

void CArena_RunTeamLogic::SDK_OnUnload()
{
	if (runTeamLogicDetour != NULL)
	{
		CDetourManager::DeleteDetour(runTeamLogicDetour);
		runTeamLogicDetour = NULL;
	}
}

bool CArena_RunTeamLogic::SDK_OnMetamodUnload(char *error, size_t maxlength)
{
	return true;
}
